@@ -1,5 +1,5 @@
 /*
- *	 wmMand-1.3.1-rc2 (C) 1999-2006 Mike Henderson (mghenderson@lanl.gov)
+ *	 wmMand-1.3.2 (C) 1999-2007 Mike Henderson (mghenderson@lanl.gov)
  * 
  *	- Mandelbrot explorer
  * 
@@ -40,6 +40,8 @@
  *		Apr 2006.
  * Version 1.3.1 - update by ciotog <wmmand@ciotog.net>
  *		Sept 2006
+ * Version 1.3.2 - update by ciotog <wmmand@ciotog.net>
+ *		Mar 2007
  */
 
 /*
@@ -88,11 +90,11 @@ typedef enum {
 	LARGEVIEWER_IM, LARGEVIEWER_XV
 } LargeViewer;
 
-#define WMMAND_VERSION "1.3.1-rc2"
+#define WMMAND_VERSION "1.3.2"
 
 void ParseCMDLine(int argc, char *argv[]);
 void ButtonPressEvent(XButtonEvent *xev, DisplayInfo *info);
-void KeyPressEvent(XKeyEvent *xev, DisplayInfo *info);
+int KeyPressEvent(XKeyEvent *xev, DisplayInfo *info);
 void ZoomEvent(XButtonEvent *lastEvent);
 void ComputeImage(double X, double Y, int width, int height, double range, unsigned char *image);
 void ComputeJulia(double X, double Y, int width, int height, double range, unsigned char *image);
@@ -136,6 +138,7 @@ int main(int argc, char *argv[]) {
 	int timer = 0;
 	int nextUpdate = autoZoomDelay;
 	int cursorHidden = False;
+	int focused = False;
 	int forceUpdate = True;
 	unsigned char *image;
 	DisplayInfo info;
@@ -167,7 +170,8 @@ int main(int argc, char *argv[]) {
 	xim->data = (char *)malloc(xim->bytes_per_line * smallImageSize);
 
 	/* create blank cursor for hiding when using keyboard */
-	blackColor.pixel = 0;	blackColor.red = 0; blackColor.blue = 0; blackColor.green = 0;
+	blackColor.pixel = 0;	blackColor.red = 0;
+	blackColor.blue = 0; blackColor.green = 0;
 	blankPixmap = XCreateBitmapFromData(display, iconwin, &blankPixmapData, 1, 1);
 	blankCursor = XCreatePixmapCursor(display, blankPixmap, blankPixmap, &blackColor, &blackColor, 0, 0);
 
@@ -184,6 +188,13 @@ int main(int argc, char *argv[]) {
 			forceUpdate = True;
 		}
 
+		if (lastEvent.type == KeyRelease && (timer + autoZoomDelay > nextUpdate)) {
+			if (cursorHidden) { /* unhide cursor */
+				XUndefineCursor(display, event.xkey.window);
+				cursorHidden = False;
+			}
+		}
+
 		/* Process any pending X events */
 		while(XPending(display)) {
 			XNextEvent(display, &event);
@@ -203,6 +214,10 @@ int main(int argc, char *argv[]) {
 						ButtonPressEvent(&event.xbutton, &info);
 						timer = 0;
 						nextUpdate = timer + 2 * autoZoomDelay; /* pause before autozoom */
+						if (!focused) {
+							XSetInputFocus(display, event.xcrossing.window, RevertToPointerRoot, CurrentTime);
+							focused = True;
+						}
 						forceUpdate = True;
 						break;
 				case ButtonRelease:
@@ -214,27 +229,26 @@ int main(int argc, char *argv[]) {
 							XDefineCursor(display, event.xkey.window, blankCursor);
 							cursorHidden = True;
 						}
-						KeyPressEvent(&event.xkey, &info);
+						if (!KeyPressEvent(&event.xkey, &info)) {
+							printf("wmMand: unhandled keypress, passing to root\n");
+							XSendEvent(display, Root, True, KeyPressMask, &event);
+						}
 						forceUpdate = True;
 						break;
-				case KeyRelease:
-						if (cursorHidden) { /* unhide cursor */
-							XUndefineCursor(display, event.xkey.window);
-							cursorHidden = False;
-						}
 				case EnterNotify:
 						printf("wmMand: pointer entered window\n");
-						/* Attempt to grab the keyboard */
-						XGrabKeyboard(display, event.xcrossing.window, True, GrabModeAsync, GrabModeAsync, CurrentTime);
 						if (info.depth == 8) XInstallColormap(display, info.cmap);
 						break;
+				case FocusOut:
+						printf("wmMand: focus out\n");
+						focused = False;
+						break;
 				case LeaveNotify:
 						printf("wmMand: pointer left window\n");
 						if (cursorHidden) { /* unhide cursor */
 							XUndefineCursor(display, event.xcrossing.window);
 							cursorHidden = False;
 						}
-						XUngrabKeyboard(display, CurrentTime);
 						if (info.depth == 8) XUninstallColormap(display, info.cmap);
 						break;
 			}
@@ -243,6 +257,11 @@ int main(int argc, char *argv[]) {
 		if (forceUpdate) {
 			/* allocate temp memory for Image */
 			image = (unsigned char *) malloc(sizeof(unsigned char) * smallImageSize * smallImageSize);
+			
+			if (image == NULL) {
+				fprintf(stderr, "wmMand: Unable to allocate memory for small image\n");
+				exit(-1);
+			}
 
 			/* create image */
 			if (fractalType == 1)
@@ -304,7 +323,7 @@ int main(int argc, char *argv[]) {
 	/* Quit program	(this code is never reached..) */
 	free(xim->data);
 	XDestroyImage(xim);
-//	XFreeCursor(display,blankCursor);
+	XFreeCursor(display,blankCursor);
 }
 
 
@@ -378,9 +397,10 @@ void ParseCMDLine(int argc, char *argv[]) {
 
 
 /*
- *	This routine handles key presses. 
+ *	This routine handles key presses.
+ *  Return value is 'True' if the key press was handled, 'False' otherwise
  */
-void KeyPressEvent(XKeyEvent *xev, DisplayInfo *info) {
+int KeyPressEvent(XKeyEvent *xev, DisplayInfo *info) {
 
 	if ((xev->keycode == XKeysymToKeycode(display, XK_z))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_Z))
@@ -400,22 +420,22 @@ void KeyPressEvent(XKeyEvent *xev, DisplayInfo *info) {
 	}
 
 	/* Arrow keys shift the center point with the zoom level as the shift factor */
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_Up))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_Up))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_KP_Up))) {
  		/* up arrow or 8 on numberpad: shift up */
 		centerY -= (zoom - 1.0) * range;
  	}
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_Down))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_Down))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_KP_Down))) {
  		/* down arrow or 2 on numberpad: shift down */
 		centerY += (zoom - 1.0) * range;
  	}
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_Left))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_Left))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_KP_Left))) {
  		/* left arrow or 4 on numberpad: shift left */
 		centerX -= (zoom - 1.0) * range;
  	}
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_Right))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_Right))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_KP_Right))) {
  		/* right arrow or 6 on numberpad: shift right */
 		centerX += (zoom - 1.0) * range;
@@ -447,46 +467,54 @@ void KeyPressEvent(XKeyEvent *xev, DisplayInfo *info) {
 
 	/* General keystrokes follow */
 
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_c))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_c))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_C))) {
 		/* c key: change color table */
 		colorTable = (colorTable + 1) % numPalettes;
 		SetColorTable(info, colorTable);
 	}
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_i))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_i))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_I))) {
 		/* i key: change iterations level */
 		iterations = (iterations + 1) % nIterList;
 		maxIterations = iterList[iterations];
 	}
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_r))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_r))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_R))) {
 		/* r key: reset view to default */
 		centerX =	0.0;
 		centerY =	0.0;
 		range = 2.0;
 	}
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_b))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_b))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_B))) {
 		/* b key: toggle button bar */
 		buttonsUp = !buttonsUp;
 	}
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_v))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_v))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_V))) {
 		/* v key: view big image with ImageMagic */
 		ViewLargeImage(centerX, centerY, largeImageSize, largeImageSize, range, info);
 	}
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_Escape))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_Escape))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_q))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_Q))) {
 		/* Esc or q key: try to release mouse grab, in case it's not releasing otherwise */
 		XUngrabPointer(display, CurrentTime);
 	}
-	else if ((xev-> keycode == XKeysymToKeycode(display, XK_x))
+	else if ((xev->keycode == XKeysymToKeycode(display, XK_x))
 			|| (xev->keycode == XKeysymToKeycode(display, XK_X))) {
 		/* x key: launch fractal program XaoS */
 		LaunchFractalViewer(centerX, centerY, largeImageSize, largeImageSize, range);
 	}
+	
+	else {
+		/* keypress wasn't handled */
+		return False;
+	}
+
+	/* keypress was handled */
+	return True;
 }
 
 
@@ -630,11 +658,23 @@ void ComputeImage(double X, double Y, int width, int height, double range, unsig
 
 	/* Initialize hist[] */
 	hist = (int *) malloc(sizeof(int) * maxIterations);
+
+	if (hist == NULL) {
+		fprintf(stderr, "wmMand: unable to allocate memory for hist\n");
+		return;
+	}
+	
 	for (i = 0; i < maxIterations; ++i) hist[i] = 0;
 
 	/* allocate memory for intImage */
 	intImage = (int *) malloc(sizeof(int) * width * height);
  
+	if (intImage == NULL) {
+		fprintf(stderr, "wmMand: unable to allocate memory for image\n");
+		free(hist);
+		return;
+	}
+ 
 	/* compute the (integer) map first */
 	nMin = 9999, nMax = -9999, nCount = 0;
 	halfWidth = width / 2.0;
@@ -738,10 +778,22 @@ double	CX, CY;
 
 	/* Initialize hist[] */
 	hist = (int *) malloc(sizeof(int) * maxIterations);
+
+	if (hist == NULL) {
+		fprintf(stderr, "wmMand: unable to allocate memory for hist\n");
+		return;
+	}
+	
 	for (i = 0; i < maxIterations; ++i) hist[i] = 0;
 
 	/* allocate memory for intImage */
 	intImage = (int *) malloc(sizeof(int) * width * height);
+
+	if (intImage == NULL) {
+		fprintf(stderr, "wmMand: unable to allocate memory for image\n");
+		free(hist);
+		return;
+	}
  
 	/* compute the (integer) map first */
 	nMin = 9999, nMax = -9999, nCount = 0;
@@ -847,9 +899,20 @@ void ViewLargeImage(double X, double Y, int width, int height, double range, Dis
 	/* allocate memory for image data */
 	image = (unsigned char *) malloc(sizeof(unsigned char) * height * width);
 
+	if (image == NULL) {
+		fprintf(stderr, "wmMand: unable to allocate memory for large image\n");
+		return;
+	}
+
 	/* allocate memory for the command string */
 	fp_command = (char *) malloc(sizeof(char)*80);
-
+	
+	if (fp_command == NULL) {
+		fprintf(stderr, "wmMand: unable to allocate memory for viewer command string\n");
+		free(image);
+		return;
+	}
+	
 	/* create large fractal */
 	if (fractalType == 1)
 		ComputeImage(X, Y, height, width, range, image);
@@ -893,17 +956,26 @@ void ViewLargeImage(double X, double Y, int width, int height, double range, Dis
 void LaunchFractalViewer(double centerX, double centerY, int width, int height, double range) {
 	char *fp_command[11];
 	pid_t fp_pid;
-	int i;
+	int i, j;
 
 	/* 
 	 * allocate memory for the command and attribute strings
 	 * the longest possible are the floating point values at 13 characters,
 	 * eg. "-1.196006e+01"
 	 */
-	for (i = 0; i < 10; ++i)
+	for (i = 0; i < 10; ++i) {
 		fp_command[i] = (char *) malloc(sizeof(char)*25);
+		
+		if (fp_command[i] == NULL) {
+			fprintf(stderr, "wmMand: Unable to allocate memory for XaoS command string\n");
+			for (j = 0; j < i; ++j)
+				free(fp_command[j]);
+			return;
+		}
+		
+	}
 
-	/* attribute list must end with a null-terminated string */
+	/* attribute list must end with a NULL-terminated string */
 	fp_command[10] = (char *) NULL;
 
 	/*
@@ -933,8 +1005,8 @@ void LaunchFractalViewer(double centerX, double centerY, int width, int height,
 	else {
 		if (fp_pid == (pid_t) 0) { // Child process, which just handles the fractal viewer
 			if (execvp(fp_command[0], fp_command) < 0) {
-				exit(EXIT_FAILURE);
 				fprintf(stderr, "wmMand: unable to launch Xaos\n");
+				exit(EXIT_FAILURE);
 			}
 			printf("wmMand: This code is never reached\n");
 			exit(EXIT_SUCCESS);
@@ -987,7 +1059,7 @@ void SetColorTable(DisplayInfo *info, int tableNumber) {
 	}
 
 	/*
-	 *	 Create an XImage with null data. Then allocate space for data. 
+	 *	 Create an XImage with NULL data. Then allocate space for data. 
 	 */
 	info->format = ZPixmap;
 
@@ -1026,7 +1098,7 @@ void SetColorTable(DisplayInfo *info, int tableNumber) {
 		info->bitmap_pad = 32;
 	}
 	else {
-		fprintf(stderr, "wmMand: Need at least 8-bit display!\n");
+		fprintf(stderr, "wmMand: Needs at least 8-bit display\n");
 		exit(-1);
 	}
 }
